//+------------------------------------------------------------------+
//|                                      FX5_Divergence_RSI_V2.1.mq4 |
//|                                                              FX5 |
//|                                                    hazem@uk2.net |
//|             modified to RSI indicator, instead of oSMA, by LaurV |
//+------------------------------------------------------------------+
#property copyright "Copyright (C) 2007, FX5"
#property link      "hazem@uk2.net"

//----
#property indicator_separate_window
#property indicator_buffers 3
#property indicator_color1 DodgerBlue
#property indicator_color2 Aqua
#property indicator_color3 DeepPink
#property indicator_level1 20
#property indicator_level2 50
#property indicator_level3 80
#property indicator_minimum 0
#property indicator_maximum 100
#property indicator_levelcolor DarkSlateGray


//---- input parameters
extern string    separator1 = "*** RSI Settings ***";
extern int       RSI_Period = 5;
extern int       LowerLimit = 20;
extern int       UpperLimit = 80;
extern string    separator2 = "*** Indicator Settings ***";
extern int       LookBack   = 50;
extern bool      drawDivergenceLines = true;
extern bool      displayAlert = true;

//---- buffers
double RSI_Line[];
double bullishDivergence[];
double bearishDivergence[];

//----
static datetime lastAlertTime;

//+------------------------------------------------------------------+
//| Custom indicator initialization function                         |
//+------------------------------------------------------------------+
int init()
  {
//---- indicators
   SetIndexStyle(0, DRAW_LINE, STYLE_SOLID, 2);
   SetIndexStyle(1, DRAW_ARROW);
   SetIndexStyle(2, DRAW_ARROW);

   SetIndexBuffer(0, RSI_Line);
   SetIndexBuffer(1, bullishDivergence);
   SetIndexBuffer(2, bearishDivergence);
//----   
   SetIndexArrow(1, 233);
   SetIndexArrow(2, 234);
//----
   IndicatorDigits(Digits + 2);
   IndicatorShortName("RSI_Divergence(" + RSI_Period + ")");
   
   return(0);
  }
  
//+------------------------------------------------------------------+
//| Custom indicator deinitialization function                       |
//+------------------------------------------------------------------+
int deinit()
  {
   for(int i = ObjectsTotal() - 1; i >= 0; i--)
     {
       string label = ObjectName(i);
       if(StringSubstr(label, 0, 14) != "DivergenceLine")
           continue;
       ObjectDelete(label);   
     }
   return(0);
  }

//+------------------------------------------------------------------+
//| Custom indicator iteration function                              |
//+------------------------------------------------------------------+
int start()
  {
   int countedBars = IndicatorCounted();
   if(countedBars < 0)
       countedBars = 0;
   CalculateIndicator(countedBars);
   return(0);
  }

//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
void CalculateIndicator(int countedBars)
  {
   for(int i = Bars - countedBars; i >= 0; i--)
     {
      CalculateRSI(i);
      CatchBullishDivergence(i + 2);
      CatchBearishDivergence(i + 2);
     }              
  }
  
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
void CalculateRSI(int i)
  {
   RSI_Line[i] = iRSI(NULL, 0, RSI_Period, PRICE_CLOSE, i);
  }


//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
void CatchBullishDivergence(int shift)
  {
   if(IsIndicatorTrough(shift) == false)
       return;
   int currentTrough = shift;
   int lastTrough = GetIndicatorLastLowerTrough(shift);
   if(RSI_Line[currentTrough] > RSI_Line[lastTrough] && Close[currentTrough] < Close[lastTrough] 
         && RSI_Line[lastTrough] < LowerLimit)
     {
      bullishDivergence[currentTrough-2] = 20; //RSI_Line[currentTrough]; align arrows
                                               //original indicator was cheating here!
      if(drawDivergenceLines == true)
        {
          DrawPriceTrendLine(Time[currentTrough], Time[lastTrough], Close[currentTrough], 
                             Close[lastTrough], Aqua, STYLE_SOLID);
          DrawIndicatorTrendLine(Time[currentTrough], Time[lastTrough], RSI_Line[currentTrough],
                                 RSI_Line[lastTrough], Aqua, STYLE_SOLID);
        }
      if(displayAlert == true)
          DisplayAlert("Classical bullish divergence on: ", currentTrough);  
     }
   if(RSI_Line[currentTrough] < RSI_Line[lastTrough] && Close[currentTrough] > Close[lastTrough]
            && RSI_Line[currentTrough] < LowerLimit)
     {
      bullishDivergence[currentTrough] = RSI_Line[currentTrough];
      if(drawDivergenceLines == true)
        {
          DrawPriceTrendLine(Time[currentTrough], Time[lastTrough], Close[currentTrough], 
                             Close[lastTrough], Aqua, STYLE_DOT);
          DrawIndicatorTrendLine(Time[currentTrough], Time[lastTrough], RSI_Line[currentTrough],
                                 RSI_Line[lastTrough], Aqua, STYLE_DOT);
        }
      if(displayAlert == true)
          DisplayAlert("Reverse bullish divergence on: ", currentTrough);   
     }      
  }
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
void CatchBearishDivergence(int shift)
  {
   if(IsIndicatorPeak(shift) == false)
       return;
   int currentPeak = shift;
   int lastPeak = GetIndicatorLastHigherPeak(shift);
   
   if(RSI_Line[currentPeak] < RSI_Line[lastPeak] && Close[currentPeak] > Close[lastPeak]
            && RSI_Line[lastPeak] > UpperLimit)
     {
       bearishDivergence[currentPeak-2] = 80;  //RSI_Line[currentPeak]; align arrows !!
                                               //original indicator was cheating here!
       if(drawDivergenceLines == true)
         {
           DrawPriceTrendLine(Time[currentPeak], Time[lastPeak], Close[currentPeak], 
                              Close[lastPeak], DeepPink, STYLE_SOLID);
           DrawIndicatorTrendLine(Time[currentPeak], Time[lastPeak], RSI_Line[currentPeak],
                                  RSI_Line[lastPeak], DeepPink, STYLE_SOLID);
         }
       if(displayAlert == true)
           DisplayAlert("Classical bearish divergence on: ", currentPeak);  
     }
   if(RSI_Line[currentPeak] > RSI_Line[lastPeak] && Close[currentPeak] < Close[lastPeak]
             && RSI_Line[currentPeak] > UpperLimit)
     {
       bearishDivergence[currentPeak] = RSI_Line[currentPeak];
       if(drawDivergenceLines == true)
         {
           DrawPriceTrendLine(Time[currentPeak], Time[lastPeak], Close[currentPeak], 
                              Close[lastPeak], DeepPink, STYLE_DOT);
           DrawIndicatorTrendLine(Time[currentPeak], Time[lastPeak], RSI_Line[currentPeak],
                                  RSI_Line[lastPeak], DeepPink, STYLE_DOT);
         }
       if(displayAlert == true)
           DisplayAlert("Reverse bearish divergence on: ", currentPeak);   
     }   
  }
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
bool IsIndicatorPeak(int shift)
  {
   if(RSI_Line[shift] > 50 && RSI_Line[shift] > RSI_Line[shift+1] && RSI_Line[shift] > RSI_Line[shift-1]
      && RSI_Line[shift] > RSI_Line[shift+2] && RSI_Line[shift] > RSI_Line[shift-2])
     {
       for(int i = shift + 1; i < shift+LookBack && i<Bars; i++)
         {
           if(RSI_Line[i] < 50)
              return(true);
           if(RSI_Line[i] > RSI_Line[shift])
              break;            
         }
     }   
   return(false);
  }
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
bool IsIndicatorTrough(int shift)
  {
   if(RSI_Line[shift] < LowerLimit && RSI_Line[shift] < RSI_Line[shift+1] && RSI_Line[shift] < RSI_Line[shift-1]
      && RSI_Line[shift] < RSI_Line[shift+2] && RSI_Line[shift] < RSI_Line[shift-2])
     {
       for(int i = shift + 1; i < shift+LookBack && i<Bars; i++)
         {
           if(RSI_Line[i] > LowerLimit)
               return(true);
           if(RSI_Line[i] < RSI_Line[shift])
               break;            
         }
     }   
   return(false);
  }
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
int GetIndicatorLastHigherPeak(int shift)
  {
    for(int i = shift + 5; i < shift+LookBack && i < Bars; i++)
      {
        if(RSI_Line[i] >= RSI_Line[i+1] && RSI_Line[i] > RSI_Line[i+2] &&
           RSI_Line[i] >= RSI_Line[i-1] && RSI_Line[i] > RSI_Line[i-2] &&
           RSI_Line[i] >= RSI_Line[shift])
            return(i);
      }
    return(-1);
  }
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
int GetIndicatorLastLowerTrough(int shift)
  {  
    for(int i = shift + 5; i < shift+LookBack && i < Bars; i++)
      {
        if(RSI_Line[i] <= RSI_Line[i+1] && RSI_Line[i] < RSI_Line[i+2] &&
           RSI_Line[i] <= RSI_Line[i-1] && RSI_Line[i] < RSI_Line[i-2] &&
           RSI_Line[i] <= RSI_Line[shift])
            return(i);
      }
    return(-1);
  }
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
void DisplayAlert(string message, int shift)
  {
   if(shift <= 2 && Time[shift] != lastAlertTime)
     {
       lastAlertTime = Time[shift];
       Alert(message, Symbol(), " , ", Period(), " minutes chart");
     }
  }
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
void DrawPriceTrendLine(datetime x1, datetime x2, double y1, 
                        double y2, color lineColor, double style)
  {
   string label = "DivergenceLine2.1# " + DoubleToStr(x1, 0);
   ObjectDelete(label);
   ObjectCreate(label, OBJ_TREND, 0, x1, y1, x2, y2, 0, 0);
   ObjectSet(label, OBJPROP_RAY, 0);
   ObjectSet(label, OBJPROP_COLOR, lineColor);
   ObjectSet(label, OBJPROP_STYLE, style);
  }
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
void DrawIndicatorTrendLine(datetime x1, datetime x2, double y1, 
                            double y2, color lineColor, double style)
  {
   int indicatorWindow = WindowFind("RSI_Divergence(" + RSI_Period + ")");
   if(indicatorWindow < 0)
       return;
   string label = "DivergenceLine2.1$# " + DoubleToStr(x1, 0);
   ObjectDelete(label);
   ObjectCreate(label, OBJ_TREND, indicatorWindow, x1, y1, x2, y2, 0, 0);
   ObjectSet(label, OBJPROP_RAY, 0);
   ObjectSet(label, OBJPROP_COLOR, lineColor);
   ObjectSet(label, OBJPROP_STYLE, style);
  }
//+------------------------------------------------------------------+



